import sys

from PySide6 import QtCore, QtWidgets
from PySide6.QtCore import Slot, QObject, Signal
from temp_humi import Ui_MainWindow
from threading import Thread
import random

from paho.mqtt import client as mqtt_client

import json

# 读取配置信息
with open("./setting.json", "r") as f:
    setting = json.load(f)
    broker = setting['broker']
    port = setting['port']
    username = setting['client']
    password = setting['passwd']
    topic = setting['topic']

# 随机生成Client ID
client_id = f'ternimal-mqtt-{random.randint(0, 100)}'


class MQTT(QObject):
    """MQTT通信"""

    signal = Signal(str, str)

    def __init__(self, parent=None):
        super().__init__(parent)

    def _connect_mqtt(self) -> mqtt_client:
        def on_connect(client, userdata, flags, rc):
            if rc == 0:
                print("Connected to MQTT Broker!")
            else:
                print("Failed to connect, return code %d\n", rc)

        client = mqtt_client.Client(client_id)
        client.username_pw_set(username=username, password=password)
        client.on_connect = on_connect
        client.connect(broker, port)
        return client

    def _subscribe(self, client: mqtt_client):
        def on_message(client, userdata, msg):
            print(f"Received `{msg.payload.decode()}` from `{msg.topic}` topic")
            self.signal.emit(msg.topic, msg.payload.decode())

        client.subscribe(topic)
        client.on_message = on_message

    def mqtt_run(self):

        client = self._connect_mqtt()
        self._subscribe(client)
        client.loop_start()  # 开启新线程处理MQTT数据

class View(QtWidgets.QWidget, Ui_MainWindow):
    def __int__(self):
        super().__init__()
        self.ctext = ''
        self.topic = 'lugl_pub'

    @Slot()
    def show_data(self, topic, payload):  # 显示订阅事件的信息
        print("topic:" + topic)
        print("payload:" + payload)
        try:
            mydata = json.loads(payload)
            self.label_TEMP.setText(mydata['temp'] + '度')
            self.label_HUMI.setText(mydata['humi'] + '%')
        except Exception as e:
            print("err:" + str(e))


    def on_connection_clicked(self):  # 连接服务端的触发函数

        def run():
            client = MQTT()  # 实例化MQTT
            client.mqtt_run()

            client.signal.connect(view.show_data)  # 将MQTT数据接收信号和UI刷新槽关联

        t = Thread(target=run)
        t.start()
        # 开启多线程，防止显示事件信息和UI界面刷新冲突导致的界面卡顿



if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    view = View()  # 对UI进行实例化
    view.setupUi(view)
    view.show()  # 显示窗口和添加的部件
    view.setWindowTitle("MQTT测试V1.0")  # 设置UI的名字
    view.on_connection_clicked()
    sys.exit(app.exec())  # 对事件进行循环
